/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "outlook/OutlookConfig.h"
#include "spds/AccessConfig.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "Client/DMTClientConfig.h"



OutlookConfig::OutlookConfig() {
}

OutlookConfig::~OutlookConfig() {
}


/*
 * Get the value of the 'propertyName' key, under the generic path 'nodeContext'.
 *
 * @param nodeContext   : relative path of the interested node in the DMTree
 * @param propertyName	: name of the source property (exact name as it is stored in the DMTree)
 * @param propertyValue : [output] the value of the property (must be allocated before the call)
 * @return				: "0" if OK,  "-1" if errors
*/
int OutlookConfig::getPropertyValue(const char* nodeContext, const char* propertyName, char* propertyValue) {
   
	int ret = 0;
	char context[DIM_MANAGEMENT_PATH];

	// Get node
    DMTree* dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
	sprintf(context, T("%s/%s"), APPLICATION_URI, nodeContext);
    ManagementNode *node = dmt->getManagementNode(context);

    if (node == NULL) {
		lastErrorCode = ERR_INVALID_CONTEXT;
		sprintf(lastErrorMsg, ERRMSG_INVALID_CONTEXT, context);
		LOG.error(lastErrorMsg);
		ret = -1;
		goto finally;
    }

    // Get property value (new is returned)
    char* tmp = node->getPropertyValue(propertyName);
	strcpy(propertyValue, tmp);
	delete [] tmp;

    // propertyValue could be a empty string
	//if (strcmp(propertyValue, "") == 0) {
	//	sprintf(lastErrorMsg, "Invalid property value: (null)");
	//	LOG.error(lastErrorMsg);
	//	ret = -1;
	//}

finally:

    if (dmt)
        delete dmt;
    if (node)
        delete node;

	return ret;
}


/*
 * Set the value of the 'propertyName' key, under the generic path 'nodeContext'.
 *
 * @param nodeContext   : relative path of the interested node in the DMTree
 * @param propertyName	: name of the source property (exact name as it is stored in the DMTree)
 * @param propertyValue : the value of the property to set
 * @return				: "0" if OK,  "-1" if errors
*/
int OutlookConfig::setPropertyValue(const char* nodeContext, const char* propertyName, const char* propertyValue) {
    
	char context[DIM_MANAGEMENT_PATH];
    int ret = 0;

	DMTree* dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
	sprintf(context, T("%s/%s"), APPLICATION_URI, nodeContext);
    ManagementNode *node = dmt->getManagementNode(context);
    
	if (node == NULL) {
		lastErrorCode = ERR_INVALID_CONTEXT;
		sprintf(lastErrorMsg, ERRMSG_INVALID_CONTEXT, context);
		LOG.error(lastErrorMsg);
		ret = -1;
		goto finally;
    }

    // Set property value.
    node->setPropertyValue(propertyName, propertyValue);

finally:

    if (dmt)
        delete dmt;
    if (node)
        delete node;

	return ret;
}




/*
 * Get the value of a specific configuration property for Outlook Client.
 * Config properties are common for all sources and are located 
 * under a specific node in DMTree (see SPDS_SYNCML).
 *
 * @param spdmName	    : property name defined for Outlook Client
 * @param propertyValue : [output] the value of the property (must be allocated before the call)
 * @return				: "0" if OK,  "-1" if errors
*/
int OutlookConfig::getConfigPropertyValue (const char* spdmName, char* propertyValue) {

	int ret = 0;
	char* propertyName = new char[MAX_KEY_LENGTH];

    if      (strcmp(spdmName, SPDM_SYNCURL) == 0)
        sprintf(propertyName, PROPERTY_SYNC_URL);
    else if (strcmp(spdmName, SPDM_SERVERNAME) == 0)
        sprintf(propertyName, PROPERTY_SERVER_NAME);
    else if (strcmp(spdmName, SPDM_PASSWORD) == 0)
        sprintf(propertyName, PROPERTY_PASSWORD);
    else if (strcmp(spdmName, SPDM_USERNAME) == 0)
        sprintf(propertyName, PROPERTY_USERNAME);
    else if (strcmp(spdmName, SPDM_DEVICEID) == 0)
        sprintf(propertyName, PROPERTY_DEVICE_ID);
    else if (strcmp(spdmName, SPDM_ENCRYPTION) == 0)
        sprintf(propertyName, PROPERTY_ENCRYPTION);
    else if (strcmp(spdmName, SPDM_USE_PROXY) == 0)
        sprintf(propertyName, PROPERTY_USE_PROXY);
    else if (strcmp(spdmName, SPDM_PROXY_USERNAME) == 0)
        sprintf(propertyName, PROPERTY_PROXY_USERNAME);
    else if (strcmp(spdmName, SPDM_PROXY_PASSWORD) == 0)
        sprintf(propertyName, PROPERTY_PROXY_PASSWORD);
	else {
		lastErrorCode = ERR_INVALID_CONTEXT;
		sprintf(lastErrorMsg, "Invalid config property name: %s", spdmName);
		LOG.error(lastErrorMsg);
		ret = -1;
		goto finally;
	}

	ret = getPropertyValue(CONTEXT_SYNCML, propertyName, propertyValue);


finally:

	delete [] propertyName;
    return ret;
}


/*
 * Set the value of a specific configuration property for Outlook Client.
 * Config properties are common for all sources and are located 
 * under a specific node in DMTree (see SPDS_SYNCML).
 *
 * @param spdmName	    : property name defined for Outlook Client
 * @param propertyValue : the value of the property to set
 * @return				: "0" if OK,  "-1" if errors
*/
int OutlookConfig::setConfigPropertyValue (const char* spdmName, const char* propertyValue) {
	
	int ret = 0;
	char* propertyName = new char[MAX_KEY_LENGTH];

    // Here chose the parameter to set.
    if      (strcmp(spdmName, SPDM_SYNCURL) == 0)
        sprintf(propertyName, PROPERTY_SYNC_URL);
    else if (strcmp(spdmName, SPDM_SERVERNAME) == 0)
        sprintf(propertyName, PROPERTY_SERVER_NAME);
    else if (strcmp(spdmName, SPDM_PASSWORD) == 0)
        sprintf(propertyName, PROPERTY_PASSWORD);
    else if (strcmp(spdmName, SPDM_USERNAME) == 0)
        sprintf(propertyName, PROPERTY_USERNAME);
    else if (strcmp(spdmName, SPDM_DEVICEID) == 0)
        sprintf(propertyName, PROPERTY_DEVICE_ID);
    else if (strcmp(spdmName, SPDM_ENCRYPTION) == 0)
        sprintf(propertyName, PROPERTY_ENCRYPTION);
    else if (strcmp(spdmName, SPDM_USE_PROXY) == 0)
        sprintf(propertyName, PROPERTY_USE_PROXY);
    else if (strcmp(spdmName, SPDM_PROXY_USERNAME) == 0)
        sprintf(propertyName, PROPERTY_PROXY_USERNAME);
    else if (strcmp(spdmName, SPDM_PROXY_PASSWORD) == 0)
        sprintf(propertyName, PROPERTY_PROXY_PASSWORD);
	else {
		lastErrorCode = ERR_INVALID_CONTEXT;
		sprintf(lastErrorMsg, "Invalid config property name: %s", spdmName);
		LOG.error(lastErrorMsg);
		ret = -1;
		goto finally;
	}

	ret = setPropertyValue(CONTEXT_SYNCML, propertyName, propertyValue);


finally:

	delete [] propertyName;
    return ret;
}




/*
 * Retrieve property value of a specific source.
 * @param sourceName	: name of the syncsource
 * @param propertyName	: name of the source property (exact name as it is stored in the DMTree)
 * @param propertyValue : [output] the value of the property
 * @return				: "0" if OK,  "-1" if errors
 */
int OutlookConfig::getSourcePropertyValue (const char* sourceName, const char* propertyName, char* propertyValue) {

	int ret = 0;
	char* sourceContext = new char[MAX_KEY_LENGTH];

    if (strcmp(sourceName, SYNC_APPOINTMENTS) == 0) 
		sprintf(sourceContext, CONTEXT_APPOINTMENTS);
    else if (strcmp(sourceName, SYNC_CONTACTS) == 0) 
		sprintf(sourceContext, CONTEXT_CONTACTS);
    else if (strcmp(sourceName, SYNC_TASKS) == 0) 
		sprintf(sourceContext, CONTEXT_TASKS);
    else if (strcmp(sourceName, SYNC_NOTES) == 0) 
		sprintf(sourceContext, CONTEXT_NOTES);
	else {
		lastErrorCode = ERR_INVALID_CONTEXT;
		sprintf(lastErrorMsg, "Invalid source name: %s", sourceName);
		LOG.error(lastErrorMsg);
		ret = -1;
		goto finally;
	}

	ret = getPropertyValue(sourceContext, propertyName, propertyValue);


finally:

    delete [] sourceContext;
    return ret;
}


/*
 * Store property value of a specific source.
 * @param sourceName	: name of the syncsource
 * @param propertyName	: name of the source property (exact name as it is stored in the DMTree)
 * @param propertyValue	: the value of the property to set
 * @return				: "0" if OK,  "-1" if errors
 */
int OutlookConfig::setSourcePropertyValue (const char* sourceName, const char* propertyName, const char* propertyValue) {
	
	int ret = 0;
	char* sourceContext = new char[MAX_KEY_LENGTH];

    if (strcmp(sourceName, SYNC_APPOINTMENTS) == 0) 
		sprintf(sourceContext, CONTEXT_APPOINTMENTS);
    else if (strcmp(sourceName, SYNC_CONTACTS) == 0) 
		sprintf(sourceContext, CONTEXT_CONTACTS);
    else if (strcmp(sourceName, SYNC_TASKS) == 0) 
		sprintf(sourceContext, CONTEXT_TASKS);
    else if (strcmp(sourceName, SYNC_NOTES) == 0) 
		sprintf(sourceContext, CONTEXT_NOTES);
	else {
		lastErrorCode = ERR_INVALID_CONTEXT;
		sprintf(lastErrorMsg, "Invalid source name: %s", sourceName);
		LOG.error(lastErrorMsg);
		ret = -1;
		goto finally;
	}

	ret = setPropertyValue(sourceContext, propertyName, propertyValue);


finally:

    delete [] sourceContext;
    return ret;
}

