/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <windows.h>
#include <comutil.h>

#include "Outlook/WindowsSyncSource.h"
#include "Outlook/winmaincpp.h"
#include "spds/SyncItemStatus.h"
#include "base/util/utils.h"
#include "base/Log.h"


/*
* Function Pointers to the CALLBACK functions implemented in VB.
*/
static CBFunction1 getAllItemsCB;			// Get items from Outlook (all items together).
static CBFunction1 getNewItemsCB;           // (All/New/Mod write to file, Del returns an xml)
static CBFunction1 getModItemsCB;
static CBFunction3 getDelItemsCB;
static CBFunction1 getAllItemsContactsCB;
static CBFunction1 getNewItemsContactsCB;
static CBFunction1 getModItemsContactsCB;
static CBFunction3 getDelItemsContactsCB;
static CBFunction3 getAllItemsKeyCB;		// Get item keys for 'Refresh-from-server' mode.
static CBFunction3 getAllItemsKeyContactsCB;
static CBFunction2 addItemCB;				// Update Outlook with items (callback for each single item).
static CBFunction4 updateItemCB;
static CBFunction2 addItemContactsCB;
static CBFunction4 updateItemContactsCB;
static CBFunction4 deleteItemCB;
static CBFunction1 prepareSyncFilesCB;		// Read Outlook folders - initial scan of all items.

// Item counters.
static int call;
static int cnew;
static int cupdated;
static int cdeleted;
static int ckey;

//---------------------------------- Static functions -------------------------------------------------
/*
 * Assign the addresses of the callback functions to static pointers of WindowsSyncSource.
 * This function MUST be called before the sync process.
 * (Now we can call these VB functions as normal function calls inside C++ code.)
 */
DWORD WINAPI setCBFunctionsAddresses(long AddressOfgetAllItemsCB,    long AddressOfgetAllItemsContactsCB, 
									 long AddressOfgetNewItemsCB,    long AddressOfgetNewItemsContactsCB,
                                     long AddressOfgetModItemsCB,    long AddressOfgetModItemsContactsCB,
									 long AddressOfgetDelItemsCB,    long AddressOfgetDelItemsContactsCB,
									 long AddressOfgetAllItemsKeyCB, long AddressOfgetAllItemsKeyContactsCB,
                                     long AddressOfaddItemCB,        long AddressOfaddItemContactsCB,
									 long AddressOfupdateItemCB,     long AddressOfupdateItemContactsCB,
									 long AddressOfdeleteItemCB,	 long AddressOfprepareSyncFiles){

    getAllItemsCB = (CBFunction1)AddressOfgetAllItemsCB;
    getNewItemsCB = (CBFunction1)AddressOfgetNewItemsCB;
    getModItemsCB = (CBFunction1)AddressOfgetModItemsCB;
    getDelItemsCB = (CBFunction3)AddressOfgetDelItemsCB;

	getAllItemsContactsCB = (CBFunction1)AddressOfgetAllItemsContactsCB;
    getNewItemsContactsCB = (CBFunction1)AddressOfgetNewItemsContactsCB;
    getModItemsContactsCB = (CBFunction1)AddressOfgetModItemsContactsCB;
    getDelItemsContactsCB = (CBFunction3)AddressOfgetDelItemsContactsCB;

	getAllItemsKeyCB         = (CBFunction3)AddressOfgetAllItemsKeyCB;
	getAllItemsKeyContactsCB = (CBFunction3)AddressOfgetAllItemsKeyContactsCB;

    addItemCB			 = (CBFunction2)AddressOfaddItemCB;
    updateItemCB		 = (CBFunction4)AddressOfupdateItemCB;
	addItemContactsCB    = (CBFunction2)AddressOfaddItemContactsCB;
    updateItemContactsCB = (CBFunction4)AddressOfupdateItemContactsCB;
    deleteItemCB		 = (CBFunction4)AddressOfdeleteItemCB;

	prepareSyncFilesCB = (CBFunction1)AddressOfprepareSyncFiles;

    return 0;
}


//------------------------------ WindowsSyncSource Methods --------------------------------------------
WindowsSyncSource::WindowsSyncSource(const wchar_t* name, wchar_t* t) : SyncSource(name) {
    c = NULL;        
    setPath(TEXT("\\"));
	setType(t);
}

WindowsSyncSource::WindowsSyncSource(const wchar_t* name) : SyncSource(name) {
    c = NULL;        
    setPath(TEXT("\\"));
}


WindowsSyncSource::~WindowsSyncSource() {
    if (c) 
        delete c; 
    c = NULL;
}


void WindowsSyncSource::setPath(const wchar_t* p) {
    if (p == NULL)
        path = wstrdup(TEXT(""));
    else
        path = wstrdup(p);
    
}

wchar_t* WindowsSyncSource::getPath() {
    return path;
}


void WindowsSyncSource::setType(wchar_t* t) {
    type = t;
}

wchar_t* WindowsSyncSource::getType() {
    return type;
}



/* ===========                                             =============
 * =========== Methods to get single items from the Client =============
 * ===========                                             =============
 */
/*
* Return the first SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
* 
* How it works:
* -------------
* - Retrieve ALL items from the client using a CALLBACK function (writes to fs)
* - Reads ALL Items (from fs) and fills the SyncItem Array
* - Returns the first Item of all
*/
SyncItem* WindowsSyncSource::getFirstItem() {

    if (c == NULL) {
        c = new Container();
    }

    // CALLBACK to Visual Basic. Writes all Items on file system
    if (wcscmp(CONTACTS, getName()) == 0)
		getAllItemsContactsCB();
	else
		getAllItemsCB();

    // Get ALL allItems from files and fill the container
    getItemsFromFileSystem(c, getPath(), getName(), ALL);
    
    if (c->getAllItemsSize() == 0) {
        return NULL;
    }
    
    ArrayList* list = c->getAllItems();

    call = 0;
    return (SyncItem*)list->get(call);
}


/*
* Return the next SyncItem of all. It is used in case of slow or refresh sync 
* and retrieve the entire data source content.
*/
SyncItem* WindowsSyncSource::getNextItem() {        
    
    if (c == NULL)
        return NULL;

    call++;
    if (call == c->getAllItemsSize()) {
        return NULL;
    }      

    ArrayList* list = c->getAllItems();

    return (SyncItem*)list->get(call);  
}


/*
* Return the first SyncItem of new. It is used in case of fast sync.
* 
* How it works:
* -------------
* - Retrieve ALL new items from the client using a CALLBACK function (writes to fs)
* - Reads ALL new Items (from fs) and fills the SyncItem Array
* - Returns the first Item of new
*/
SyncItem* WindowsSyncSource::getFirstNewItem() {

    if (c == NULL) {
        c = new Container();
    }


    // CALLBACK to Visual Basic. Writes all new Items on file system
	if (wcscmp(CONTACTS, getName()) == 0)
		getNewItemsContactsCB();
	else
		getNewItemsCB();

    // Get ALL new Items from files and fill the container
    getItemsFromFileSystem(c, getPath(), getName(), NEW);
    
    ArrayList* list = c->getNewItems();

    cnew = 0;
    return (SyncItem*)list->get(cnew);
}

/*
* Return the next SyncItem of new. It is used in case of fast sync.
*/
SyncItem* WindowsSyncSource::getNextNewItem() {    
    
    if (c == NULL)
        return NULL;

    cnew++;
    if (cnew == c->getNewItemsSize()) {
        return NULL;
    }      

    ArrayList* list = c->getNewItems();

    return (SyncItem*)list->get(cnew);  
}


/*
* Return the first SyncItem of updated. It is used in case of fast sync.
* 
* How it works:
* -------------
* - Retrieve ALL updated items from the client using a CALLBACK function (writes to fs)
* - Reads ALL updated Items (from fs) and fills the SyncItem Array
* - Returns the first Item of updated
*/
SyncItem* WindowsSyncSource::getFirstUpdatedItem() {
    
    if (c == NULL) {
        c = new Container();
    }

    // CALLBACK to Visual Basic. Writes all updated Items on file system
    if (wcscmp(CONTACTS, getName()) == 0)
		getModItemsContactsCB();
	else
		getModItemsCB();

    // Get ALL updated Items from files and fill the container
    getItemsFromFileSystem(c, getPath(), getName(), MOD);
    
    ArrayList* list = c->getUpdatedItems();

    cupdated = 0;
    return (SyncItem*)list->get(cupdated);
}

/*
* Return the next SyncItem of updated. It is used in case of fast sync.
*/
SyncItem* WindowsSyncSource::getNextUpdatedItem() {    
    
    if (c == NULL)
        return NULL;


    cupdated++;
    if (cnew == c->getUpdatedItemsSize()) {
        return NULL;
    }      

    ArrayList* list = c->getUpdatedItems();

    return (SyncItem*)list->get(cupdated);          
}

/*
* Return the first SyncItem of deleted. It is used in case of fast sync.
* 
* How it works:
* -------------
* - Retrieve ALL deleted items from the client using a CALLBACK function (return xml string)
* - Reads ALL deleted Items (from xml string) and fills the SyncItem Array
* - Returns the first Item of deleted
*/
SyncItem* WindowsSyncSource::getFirstDeletedItem() {
    
    if (c == NULL) {
        c = new Container();
    }

    // CALLBACK to Visual Basic. Returns an xml string for deleted items.
    BSTR delXml;
	if (wcscmp(CONTACTS, getName()) == 0)
		delXml = getDelItemsContactsCB();
	else
		delXml = getDelItemsCB();

    // Fills the container with Items from the xml string.
	getDelItemsFromString (c, (wchar_t*)delXml);
    
    ArrayList* list = c->getDeletedItems();

    cdeleted = 0;
    return (SyncItem*)list->get(cdeleted);  
}

/*
* Return the next SyncItem of deleted. It is used in case of fast sync.
*/
SyncItem* WindowsSyncSource::getNextDeletedItem() {    
    
    if (c == NULL)
        return NULL;

    cdeleted++;
    if (cnew == c->getDeletedItemsSize()) {
        return NULL;
    }      

    ArrayList* list = c->getDeletedItems();

    return (SyncItem*)list->get(cdeleted);    
}


/*
 * Return the key of the first Item of All, 
 * for refresh-from-server sync.
 */
SyncItem* WindowsSyncSource::getFirstItemKey() { 
    
    if (c == NULL) {
        c = new Container();
    }

    // CALLBACK to Visual Basic. Returns an xml string with item keys
	BSTR keyXml;
    if (wcscmp(CONTACTS, getName()) == 0)
		keyXml = getAllItemsKeyContactsCB();
	else
		keyXml = getAllItemsKeyCB();

	// Fills the container with ALL items from the xml string.
	getAllItemsKeyFromString (c, (wchar_t*)keyXml);

    if (c->getAllItemsSize() == 0) {
        return NULL;
    }

    ckey = 0;
    ArrayList* list = c->getAllItems();

    return (SyncItem*)list->get(ckey);
}

/*
 * Return the key of the next Item of All, 
 * for refresh-from-server sync.
 */  
SyncItem* WindowsSyncSource::getNextItemKey() {  
    
    if (c == NULL)
        return NULL; 

    ckey ++;
    if (ckey >= c->getAllItemsSize()) {
        return NULL;
    } 

    ArrayList* list = c->getAllItems();

    return (SyncItem*)list->get(ckey);
}



/* ===========                                           =============
 * =========== Methods to set single items on the Client =============
 * ===========                                           =============
 */
/*
 * Add the item from the server to the client.
 * Sets the SyncItem key
 */
int WindowsSyncSource::addItem(SyncItem& item) {
    
    wchar_t* returnXml = NULL;
    BSTR retVB;
	int ret;
    
    // Builds an appropriate xml string for this item.
	returnXml = manageNewItem(&item, getName());

    if (!returnXml)
        return STC_NO_CONTENT;

    // Convert the wchar_t* into a BSTR (needed by VB)
    _bstr_t xmlVB = _bstr_t(returnXml);
   

    // CALLBACK to Visual Basic. Insert the Item into OL.
	// Return the LUID, <0 in case of errors.
	if (wcscmp(CONTACTS, getName()) == 0)
		retVB = addItemContactsCB((BSTR)xmlVB);
    else
		retVB = addItemCB((BSTR)xmlVB);
	
    
    // Added a new item: we need to set the LUID for mappings
	wchar_t luid[256];
	wsprintf(luid, TEXT("%s"), retVB);

    //MessageBox (NULL, luid, TEXT ("LUID of new item"), MB_SETFOREGROUND |MB_OK);

    if (wcscmp(luid, TEXT("-1")) != 0) {
		ret = STC_ITEM_ADDED;
        item.setKey(luid);
    }
	else 
		ret = STC_COMMAND_FAILED;  // command Failed;

    if (returnXml != NULL){
        delete [] returnXml; returnXml = NULL;
    }

    return ret;
}

/*
 * Update the item from the server to the client.
 */
int WindowsSyncSource::updateItem(SyncItem& item) {
    
    wchar_t* returnXml = NULL;
	int ret = STC_OK;
	DWORD retVB;
    
    // Builds an appropriate xml string for this item.
    returnXml = manageUpdatedItem(&item, getName());

    if (!returnXml)
        return STC_NO_CONTENT;

    //To convert the wchar_t* into a BSTR (needed by VB)
    _bstr_t xmlVB = _bstr_t(returnXml);
    

    // CALLBACK to Visual Basic. Updates the Item into OL.
	if (wcscmp(CONTACTS, getName()) == 0)
		retVB = updateItemContactsCB((BSTR)xmlVB);
	else
		retVB = updateItemCB((BSTR)xmlVB);

	if (retVB != 1)
		ret = STC_COMMAND_FAILED;  // command Failed;

    if (returnXml != NULL){
        delete [] returnXml; returnXml = NULL;
    }

    return ret;
}

/*
 * Delete the item from the server to the client.
 */
int WindowsSyncSource::deleteItem(SyncItem& item) {
   
    wchar_t* returnXml = NULL;
    int ret = STC_OK;
	DWORD retVB;
    
    // Builds an appropriate xml string for this item.
    returnXml = manageDeletedItem(&item);

    //To convert the wchar_t* into a BSTR (needed by VB)
    _bstr_t xmlVB = _bstr_t(returnXml);
    

    // CALLBACK to Visual Basic. Updates the Item into OL.
	retVB = deleteItemCB((BSTR)xmlVB);

   if (retVB != 1)
		ret = STC_COMMAND_FAILED;  // command Failed;

    if (returnXml != NULL){
        delete [] returnXml; returnXml = NULL;
    }

    return ret;
}

int WindowsSyncSource::beginSync() {

	// CALLBACK to VB: scan all Outlook items and prepare files for sync.
	prepareSyncFilesCB();

    return 0;
}

int WindowsSyncSource::endSync() {    
    //
    // Nothing to do
    //
    return 0;
}


void WindowsSyncSource::setItemStatus(const wchar_t* key, int status) {
    char tmp[256];
    sprintf(tmp, T("key: %ls, status: %i"), key, status);
    LOG.debug(tmp);
}


void WindowsSyncSource::assign(WindowsSyncSource& s) {
    setSyncMode(s.getSyncMode());
    setLastSync(s.getLastSync());
    setNextSync(s.getNextSync());
    setRemoteURI(s.getRemoteURI());
    setType(s.getType());
    setLastAnchor(s.getLastAnchor());
    setNextAnchor(s.getNextAnchor());
    setErrorHandler(s.getErrorHandler());
    setFilter(s.getFilter());
    setPath(getPath());
    setType(getType());
}


ArrayElement* WindowsSyncSource::clone() {
    WindowsSyncSource* s = new WindowsSyncSource(getName(), getType());
    s->setPath(getPath());

    s->assign(*this);

    return s;
}

